;; Copyright 2009, 2010, 2017 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-overlay-is-localedef-overlay-p (overlay)
  (overlay-get overlay 'localedef-overlay))

(defun my-localedef-overlay-overlay-at-p (pos)
  (memq t (mapcar 'my-overlay-is-localedef-overlay-p (overlays-at pos))))

(defun my-string-make-unibyte (str)
  (if (fboundp 'string-make-unibyte)
      (string-make-unibyte str)
    str))

(defconst my-unicode-bullet-str
  (or (condition-case nil
          (string (decode-char 'ucs 8226)) ;; U+2022
        (error nil))
      (condition-case nil
          (decode-coding-string (my-string-make-unibyte (string 226 128 162))
                                'utf-8)
        (error nil))))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count   0)
        (tempbuf "*t-localedef-overlay--temp*"))
    (switch-to-buffer tempbuf)
    (erase-buffer)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(localedef-overlay-mode))


    ;;-------------------------------------------------------------------------
    ;; localedef-overlay-mode -- buffer-local after-change-functions

    (setq after-change-functions nil)
    (let ((want after-change-functions))
      (with-temp-buffer
        (localedef-overlay-mode 1))
      (let ((got after-change-functions))
        (unless (equal got want)
          (error "localedef-overlay-mode changes global after-change-functions: got %S want %S" got want)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; autoloaded mode func

    (localedef-overlay-mode 1)
    (unless (equal t localedef-overlay-mode)
      (error "localedef-overlay-mode doesn't set localedef-overlay-mode variable"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; localedef-overlay-mode - disable

    (localedef-overlay-mode 0)
    (unless (equal nil localedef-overlay-mode)
      (error "localedef-overlay-mode 0 doesn't set localedef-overlay-mode variable"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; localedef-overlay-mode - enable

    (localedef-overlay-mode 1)
    (unless (equal t localedef-overlay-mode)
      (error "localedef-overlay-mode 1 doesn't set localedef-overlay-mode variable"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; localedef-overlay-decode-ucs

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (localedef-overlay-decode-ucs str)))
                (unless (equal got want)
                  (error "localedef-overlay-decode-ucs %S got %S want %S"
                         str got want))
                (setq count (1+ count))))

            `(("0058" "X")
              ("00F7" ;; division sign
               ,(decode-coding-string (string 247) 'iso-8859-1))))

    (if (null (localedef-overlay-decode-ucs "2022"))
        (message "skip, no unicode decode")

      (mapcar (lambda (elem)
                (let* ((str  (nth 0 elem))
                       (want (nth 1 elem))
                       (got  (localedef-overlay-decode-ucs str)))
                  (unless (equal got want)
                    (error "localedef-overlay-decode-ucs %S got %S[%s] want %S[%s]"
                           str
                           got (length got)
                           want (length want)))
                  (setq count (1+ count))))

              `(("2022" ,my-unicode-bullet-str))))  ;; bullet

    ;;-------------------------------------------------------------------------
    ;; overlay -- initial

    (localedef-overlay-mode 0)
    (erase-buffer)
    (insert "<U0058>")
    (localedef-overlay-mode 1)

    (if (null (localedef-overlay-decode-ucs "0058"))
        (message "skip, no unicode decode")

      (unless (my-localedef-overlay-overlay-at-p (point-min))
        (error "no overlay on <U> initial enable"))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; overlay -- on insert

    (localedef-overlay-mode 0)
    (erase-buffer)
    (localedef-overlay-mode 1)
    (insert "<U0058>")

    (if (null (localedef-overlay-decode-ucs "0058"))
        (message "skip, no unicode decode")

      (unless (my-localedef-overlay-overlay-at-p (point-min))
        (error "no overlay on <U> insert"))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (erase-buffer)
      (insert "<U0058>")
      (localedef-overlay-mode 1)
      (if (null (localedef-overlay-decode-ucs "0058"))
          (message "skip, no unicode decode")
        (unless (my-localedef-overlay-overlay-at-p (point-min))
          (error "oops, no overlay on <U>")))

      (unload-feature 'localedef-overlay t)

      (when (my-localedef-overlay-overlay-at-p (point-min))
        (error "unload-feature leaves overlay on <U>"))
      (when (and (listp after-change-functions)
                 (memq 'localedef-overlay-after-change after-change-functions))
        (error "unload-feature leftover after-change-functions %S"
               after-change-functions))
      (setq count (1+ count)))

    ;; couple of cycles
    (require 'localedef-overlay)
    (unload-feature 'localedef-overlay t)
    (require 'localedef-overlay)
    (unload-feature 'localedef-overlay t)
    (setq count (1+ count))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'localedef-overlay))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))


    (bury-buffer)
    (message "  %d tests" count)
    count))
